package com.norconex.blog.facebook.crawler;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashSet;
import java.util.Scanner;
import java.util.Set;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;

import com.norconex.collector.http.url.ILinkExtractor;
import com.norconex.collector.http.url.Link;
import com.norconex.commons.lang.file.ContentType;

/**
 * Extracts and returns "link" URLs for each posts.
 * @author Pascal Essiembre
 */
public class FacebookLinkExtractor implements ILinkExtractor {

    // This pattern catches the last pagination "next" element.
    private static final Pattern NEXT_PATTERN = Pattern.compile(
            "\"next\":\"([^\"]+?/posts\\?.*?)\"");
    private static final int NEXT_PATTERN_GROUP_INDEX = 1;
    
    @Override
    public Set<Link> extractLinks(InputStream content, String reference,
            ContentType contentType) throws IOException {

        Set<Link> links = new HashSet<Link>();
        
        // Extract "next" link from paging on Facebook JSON posts
        // If more than one is present, the last one will be picked
        Scanner scanner = new Scanner(content);
        String nextURL = null;
        while (scanner.findInLine(NEXT_PATTERN) != null) {
            nextURL = scanner.match().group(NEXT_PATTERN_GROUP_INDEX);
        }
        scanner.close();
        if (StringUtils.isNotBlank(nextURL)) {
            nextURL = StringEscapeUtils.unescapeEcmaScript(nextURL);
            nextURL = nextURL.replace("|", "%7C");
            Link link = new Link(nextURL);
            links.add(link);
        }
        
        // Optionally change/add to the logic to get other URLs from individual 
        // posts, such as those in the "link" JSON elements, or the /comments
        // URLs, etc.

        return links;
    }

    @Override
    public boolean accepts(String reference, ContentType contentType) {
        // Only extract URLs from Facebook JSON posts
        return FacebookUtils.isPosts(reference);
    }
}
